#if !defined LEVELS_H
#define LEVELS_H

#include <aes.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "getline.h"

struct level {
	char * title;
	char * definition;

	struct level * next;
};

struct level_data {
	char * title; /* name of this set of data */
	int size; /* number of levels in list */
	struct level * levels; /* pointer to level list */
};

struct level_data * leveldata_create (void);
void leveldata_free (struct level_data * ld);
void leveldata_clear (struct level_data * ld);
int leveldata_load (struct level_data * ld, char * title, char * filename, int win_handle);
int leveldata_classic (struct level_data * ld);
void leveldata_add_level (struct level_data * ld, char * title, char * definition);
struct level * leveldata_nth (struct level_data * ld, int n);

/* definition of builtin levels:
   these are the original levels, from Thinking Rabbit
 */

char * classic_levels[] = {
	"    #####\n    #   #\n    #$  #\n  ###  $##\n  #  $ $ #\n### # ## #   ######\n#   # ## #####  ..#\n# $  $          ..#\n##### ### #@##  ..#\n    #     #########\n    #######",
	"###########\n#..  #     ###\n#..  # $  $  #\n#..  #$####  #\n#..    @ ##  #\n#..  # #  $ ##\n###### ##$ $ #\n  # $  $ $ $ #\n  #    #     #\n  ############\n",
	"        ########\n        #     @#\n        # $#$ ##\n        # $  $#\n        ##$ $ #\n######### $ # ###\n#....  ## $  $  #\n##...    $  $   #\n#....  ##########\n########\n",
	"           ########\n           #  ....#\n############  ....#\n#    #  o o   ....#\n# ooo#o  o #  ....#\n#  o     o #  ....#\n# oo #o o o########\n#  o #     #\n## #########\n#    #    ##\n#     o   ##\n#  oo#oo  @#\n#    #    ##\n###########",
	"        #####\n        #   #####\n        # #o##  #\n        #     o #\n######### ###   #\n#....  ## o  o###\n#....    o oo ##\n#....  ##o  o @#\n#########  o  ##\n        # o o  #\n        ### ## #\n          #    #\n          ######",
	"######  ###\n#..  # ##@##\n#..  ###   #\n#..     oo #\n#..  # # o #\n#..### # o #\n#### o #o  #\n   #  o# o #\n   # o  o  #\n   #  ##   #\n   #########",
	"       #####\n #######   ##\n## # @## oo #\n#    o      #\n#  o  ###   #\n### #####o###\n# o  ### ..#\n# o o o ...#\n#    ###...#\n# oo # #...#\n#  ### #####\n####",
	"  ####\n  #  ###########\n  #    o   o o #\n  # o# o #  o  #\n  #  o o  #    #\n### o# #  #### #\n#@#o o o  ##   #\n#    o #o#   # #\n#   o    o o o #\n#####  #########\n  #      #\n  #      #\n  #......#\n  #......#\n  #......#\n  ########",
	"          #######\n          #  ...#\n      #####  ...#\n      #      . .#\n      #  ##  ...#\n      ## ##  ...#\n     ### ########\n     # ooo ##\n #####  o o #####\n##   #o o   #   #\n#@ o  o    o  o #\n###### oo o #####\n"
"     #      #\n     ########",
	" ###  #############\n##@####       #   #\n# oo   oo  o o ...#\n#  ooo#    o  #...#\n# o   # oo oo #...#\n###   #  o    #...#\n#     # o o o #...#\n#    ###### ###...#\n## #  #  o o  #...#\n#  ## # oo o o##..#\n"
"# ..# #  o      #.#\n# ..# # ooo ooo #.#\n##### #       # #.#\n    # ######### #.#\n    #           #.#\n    ###############",
	"          ####\n     #### #  #\n   ### @###o #\n  ##      o  #\n ##  o oo## ##\n #  #o##     #\n # # o oo # ###\n #   o #  # o #####\n####    #  oo #   #\n#### ## o         #\n#.    ###  ########\n#.. ..# ####\n#...#.#\n#.....#\n#######",
	"################\n#              #\n# # ######     #\n# #  o o o o#  #\n# #   o@o   ## ##\n# #  o o o###...#\n# #   o o  ##...#\n# ###ooo o ##...#\n#     # ## ##...#\n#####   ## ##...#\n    #####     ###\n        #     #\n        #######",
	"   #########\n  ##   ##  #####\n###     #  #    ###\n#  o #o #  #  ... #\n# # o#@o## # #.#. #\n#  # #o  #    . . #\n# o    o # # #.#. #\n#   ##  ##o o . . #\n# o #   #  #o#.#. #\n## o  o   o  o... #\n #o ######    ##  #\n #  #    ##########\n"
"####",
	"       #######\n #######     #\n #     # o@o #\n #oo #   #########\n # ###......##   #\n #   o......## # #\n # ###......     #\n##   #### ### #o##\n#  #o   #  o  # #\n#  o ooo  # o## #\n#   o o ###oo # #\n"
"#####     o   # #\n    ### ###   # #\n      #     #   #\n      ########  #\n             ####",
	"   ########\n   #   #  #\n   #  o   #\n ### #o   ####\n #  o  ##o   #\n #  # @ o # o#\n #  #      o ####\n"
" ## ####o##     #\n # o#.....# #   #\n #  o..**. o# ###\n##  #.....#   #\n#   ### #######\n# oo  #  #\n#  #     #\n######   #\n     #####",
	"#####\n#   ##\n#    #  ####\n# o  ####  #\n#  oo o   o#\n###@ #o    ##\n #  ##  o o ##\n # o  ## ## .#\n #  #o##o  #.#\n ###   o..##.#\n  #    #.*...#\n  # oo #.....#\n  #  #########\n  #  #\n  ####",
"   ##########\n   #..  #   #\n   #..      #\n   #..  #  ####\n  #######  #  ##\n  #            #\n  #  #  ##  #  #\n#### ##  #### ##\n#  o  ##### #  #\n# # o  o  # o  #\n# @o  o   #   ##\n#### ## #######\n   #    #\n   ######",
	"     ###########\n     #  .  #   #\n     # #.    @ #\n ##### ##..# ####\n##  # ..###     ###\n# o #...   o #  o #\n#    .. ##  ## ## #\n####o##o# o #   # #\n  ## #    #o oo # #\n  #  o # #  # o## #\n  #               #\n"
"  #  ###########  #\n  ####         ####",
	"  ######\n  #   @####\n##### o   #\n#   ##    ####\n# o #  ##    #\n# o #  ##### #\n## o  o    # #\n## o o ### # #\n## #  o  # # #\n"
"## # #o#   # #\n## ###   # # ######\n#  o  #### # #....#\n#    o    o   ..#.#\n####o  o# o   ....#\n#       #  ## ....#\n###################",
	"    ##########\n#####        ####\n#     #   o  #@ #\n# #######o####  ###\n# #    ## #  #o ..#\n# # o     #  #  #.#\n# # o  #     #o ..#\n# #  ### ##     #.#\n# ###  #  #  #o ..#\n"
"# #    #  ####  #.#\n# #o   o  o  #o ..#\n#    o # o o #  #.#\n#### o###    #o ..#\n   #    oo ###....#\n   #      ## ######\n   ########",
	"#########\n#       #\n#       ####\n## #### #  #\n## #@##    #\n# ooo o  oo#\n#  # ## o  #\n#  # ##  o ####\n####  ooo o#  #\n #   ##   ....#\n # #   # #.. .#\n #   # # ##...#\n ##### o  #...#\n     ##   #####\n      #####",
	"######     ####\n#    #######  #####\n#   o#  #  o  #   #\n#  o  o  o # o o  #\n##o o   # @# o    #\n#  o ########### ##\n# #   #.......# o#\n# ##  # ......#  #\n# #   o........o #\n# # o #.... ..#  #\n#  o o####o#### o#\n"
"# o   ### o   o  ##\n# o     o o  o    #\n## ###### o ##### #\n#         #       #\n###################",
	"    #######\n    #  #  ####\n##### o#o #  ##\n#.. #  #  #   #\n#.. # o#o #  o####\n#.  #     #o  #  #\n#..   o#  # o    #\n#..@#  #o #o  #  #\n#.. # o#     o#  #\n#.. #  #oo#o  #  ##\n#.. # o#  #  o#o  #\n#.. #  #  #   #   #\n"
"##. ####  #####   #\n ####  ####   #####",
	"###############\n#..........  .####\n#..........oo.#  #\n###########o #   ##\n#      o  o     o #\n## ####   #  o #  #\n#      #   ##  # ##\n#  o#  # ##  ### ##\n# o #o###    ### ##\n###  o #  #  ### ##\n"
"###    o ## #  # ##\n # o  #  o  o o   #\n #  o  o#ooo  #   #\n #  #  o      #####\n # @##  #  #  #\n ##############",
	"####\n#  ##############\n#  #   ..#......#\n#  # # ##### ...#\n##o#    ........#\n#   ##o######  ####\n# o #     ######@ #\n##o # o   ######  #\n#  o #ooo##       #\n#      #    #o#o###\n"
"# #### #ooooo    #\n# #    o     #   #\n# #   ##        ###\n# ######o###### o #\n#        #    #   #\n##########    #####",
	" #######\n #  #  #####\n##  #  #...###\n#  o#  #...  #\n# o #oo ...  #\n#  o#  #... .#\n#   # o########\n##o       o o #\n##  #  oo #   #\n ######  ##oo@#\n      #      ##\n      ########",
	" #################\n #...   #    #   ##\n##.....  o## # #o #\n#......#  o  #    #\n#......#  #  # #  #\n######### o  o o  #\n  #     #o##o ##o##\n ##   o    # o    #\n #  ## ### #  ##o #\n # o oo     o  o  #\n # o    o##o ######\n"
" #######  @ ##\n       ######",
	"         #####\n     #####   #\n    ## o  o  ####\n##### o  o o ##.#\n#       oo  ##..#\n#  ###### ###.. #\n## #  #    #... #\n# o   #    #... #\n#@ #o ## ####...#\n####  o oo  ##..#\n   ##  o o  o...#\n    # oo  o #  .#\n"
"    #   o o  ####\n    ######   #\n         #####",
	"#####\n#   ##\n# o  #########\n## # #       ######\n## #   o#o#@  #   #\n#  #      o #   o #\n#  ### ######### ##\n#  ## ..*..... # ##\n## ## *.*..*.* # ##\n# o########## ##o #\n#  o   o  o    o  #\n#  #   #   #   #  #\n###################",
	"       ###########\n       #   #     #\n#####  #     o o #\n#   ##### o## # ##\n# o ##   # ## o  #\n# o  @oo # ##ooo #\n## ###   # ##    #\n## #   ### #####o#\n## #     o  #....#\n#  ### ## o #....##\n# o   o #   #..o. #\n"
"#  ## o #  ##.... #\n#####   ######...##\n    #####    #####",
	"  ####\n  #  #########\n ##  ##  #   #\n #  o# o@o   ####\n #o  o  # o o#  ##\n##  o## #o o     #\n#  #  # #   ooo  #\n# o    o  o## ####\n# o o #o#  #  #\n##  ###  ###o #\n #  #....     #\n ####......####\n   #....####\n   #...##\n"
"   #...#\n   #####",
	"      ####\n  #####  #\n ##     o#\n## o  ## ###\n#@o o # o  #\n#### ##   o#\n #....#o o #\n #....#   o#\n #....  oo ##\n #... # o   #\n ######o o  #\n      #   ###\n      #o ###\n      #  #\n      ####",
	" ###########\n #     ##  #\n #   o   o #\n#### ## oo #\n#   o #    #\n# ooo # ####\n#   # # o ##\n#  #  #  o #\n# o# o#    #\n#   ..# ####\n####.. o #@#\n#.....# o# #\n##....#  o #\n ##..##    #\n  ##########",
	" #########\n #....   ##\n #.#.#  o ##\n##....# # @##\n# ....#  #  ##\n#     #o ##o #\n## ###  o    #\n #o  o o o#  #\n # #  o o ## #\n #  ###  ##  #\n #    ## ## ##\n #  o #  o  #\n"
" ###o o   ###\n   #  #####\n   ####",
	"############ ######\n#   #    # ###....#\n#   oo#   @  .....#\n#   # ###   # ....#\n## ## ###  #  ....#\n # o o     # # ####\n #  o o##  #      #\n#### #  #### # ## #\n#  # #o   ## #    #\n# o  o  # ## #   ##\n# # o o    # #   #\n"
"#  o ## ## # #####\n# oo     oo  #\n## ## ### o  #\n #    # #    #\n ###### ######",
	"            #####\n#####  ######   #\n#   ####  o o o #\n# o   ## ## ##  ##\n#   o o     o  o #\n### o  ## ##     ##\n  # ##### #####oo #\n ##o##### @##     #\n # o  ###o### o  ##\n # o  #   ###  ###\n"
" # oo o #   oo #\n #     #   ##  #\n #######.. .###\n    #.........#\n    #.........#\n    ###########",
	"###########\n#......   #########\n#......   #  ##   #\n#..### o    o     #\n#... o o #   ##   #\n#...#o#####    #  #\n###    #   #o  #o #\n  #  oo o o  o##  #\n  #  o   #o#o ##o #\n  ### ## #    ##  #\n   #  o o ## ######\n"
"   #    o  o  #\n   ##   # #   #\n    #####@#####\n        ###",
	"      ####\n####### @#\n#     o  #\n#   o## o#\n##o#...# #\n # o...  #\n # #. .# ##\n #   # #o #\n #o  o    #\n #  #######\n ####",
	"             ######\n #############....#\n##   ##     ##....#\n#  oo##  o @##....#\n#      oo o#  ....#\n#  o ## oo # # ...#\n#  o ## o  #  ....#\n## ##### ### ##.###\n##   o  o ##   .  #\n# o###  # ##### ###\n"
"#   o   #       #\n#  o #o o o###  #\n# ooo# o   # ####\n#    #  oo #\n######   ###\n     #####",
	"    ############\n    #          ##\n    #  # #oo o  #\n    #o #o#  ## @#\n   ## ## # o # ##\n   #   o #o  # #\n   #   # o   # #\n   ## o o   ## #\n   #  #  ##  o #\n   #    ## oo# #\n"
"######oo   #   #\n#....#  ########\n#.#... ##\n#....   #\n#....   #\n#########",
	"           #####\n          ##   ##\n         ##     #\n        ##  oo  #\n       ## oo  o #\n       # o    o #\n####   #   oo #####\n#  ######## ##    #\n#.            ooo@#\n#.# ####### ##   ##\n"
"#.# #######. #o o##\n#........... #    #\n##############  o #\n             ##  ##\n              ####",
	"     ########\n  ####      ######\n  #    ## o o   @#\n  # ## ##o#o o o##\n### ......#  oo ##\n#   ......#  #   #\n# # ......#o  o  #\n# #o...... oo# o #\n#   ### ###o  o ##\n"
"###  o  o  o  o #\n  #  o  o  o  o #\n  ######   ######\n       #####",
	"        #######\n    #####  #  ####\n    #   #   o    #\n #### #oo ## ##  #\n##      # #  ## ###\n#  ### o#o  o  o  #\n#...    # ##  #   #\n#...#    @ # ### ##\n#...#  ###  o  o  #\n######## ##   #   #\n          #########",
	" #####\n #   #\n # # #######\n #      o@######\n # o ##o ###   #\n # #### o    o #\n # ##### #  #o ####\n##  #### ##o      #\n#  o#  o  # ## ## #\n#         # #...# #\n"
"######  ###  ...  #\n     #### # #...# #\n          # ### # #\n          #       #\n          #########",
	"##### ####\n#...# #  ####\n#...###  o  #\n#....## o  o###\n##....##   o  #\n###... ## o o #\n# ##    #  o  #\n#  ## # ### ####\n# o # #o  o    #\n#  o @ o    o  #\n"
"#   # o oo o ###\n#  ######  ###\n# ##    ####\n###",
	"##########\n#        ####\n# ###### #  ##\n# # o o o  o #\n#       #o   #\n###o  oo#  ###\n  #  ## # o##\n  ##o#   o @#\n   #  o o ###\n   # #   o  #\n   # ##   # #\n  ##  ##### #\n  #         #\n  #.......###\n  #.......#\n  #########",
	"         ####\n #########  ##\n##  o      o #####\n#   ## ##   ##...#\n# #oo o oo#o##...#\n# #   @   #   ...#\n#  o# ###oo   ...#\n# o  oo  o ##....#\n###o       #######\n  #  #######\n  ####",
	"  #########\n  #*.*#*.*#\n  #.*.*.*.#\n  #*.*.*.*#\n  #.*.*.*.#\n  #*.*.*.*#\n  ###   ###\n    #   #\n###### ######\n#           #\n# o o o o o #\n"
"## o o o o ##\n #o o o o o#\n #   o@o   #\n #  #####  #\n ####   ####",
	"       ####\n       #  ##\n       #   ##\n       # oo ##\n     ###o  o ##\n  ####    o   #\n###  # #####  #\n#    # #....o #\n# #   o ....# #\n#  o # #.*..# #\n"
"###  #### ### #\n  #### @o  ##o##\n     ### o     #\n       #  ##   #\n       #########",
	"      ############\n     ##..    #   #\n    ##..* o    o #\n   ##..*.# # # o##\n   #..*.# # # o  #\n####...#  #    # #\n#  ## #          #\n# @o o ###  #   ##\n# o   o   # #   #\n###oo   # # # # #\n"
"  #   o   # # #####\n  # o# #####      #\n  #o   #   #    # #\n  #  ###   ##     #\n  #  #      #    ##\n  ####      ######",
	NULL /* must finish with NULL */
};

#endif
